import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useContext } from 'react';
import { useBBoxObserver } from 'react-d3-utils';
import { bboxContext } from '../../bboxContext';
import HorizontalAxisGridLines from './HorizontalAxisGridLines';
import HorizontalAxisLabel from './HorizontalAxisLabel';
import HorizontalAxisLine from './HorizontalAxisLine';
import { Ticks } from './Ticks';
export default function HorizontalAxis(props) {
    const { primaryTickStyle, primaryTickLength, tickPosition, hiddenTicks, primaryGridLineStyle, lineStyle, hiddenLine, hidden, plotWidth, plotHeight, displayPrimaryGridLines, label, labelStyle, axisRef, primaryTicks, position, tickLabelStyle, } = props;
    const isBottom = position === 'bottom';
    const textOffset = 0;
    const transform = isBottom ? `translate(0, ${plotHeight})` : undefined;
    const ticks = useBBoxObserver();
    function getTickPosition(value) {
        const { y1, y2, textPosition } = getTickY();
        return {
            line: {
                x1: value,
                x2: value,
                y1: y1,
                y2: y2,
            },
            text: {
                x1: value,
                y1: isBottom ? textPosition : -textPosition,
            },
        };
    }
    function getTickY() {
        const y = isBottom ? primaryTickLength : -primaryTickLength;
        switch (tickPosition) {
            case 'center':
                return {
                    y1: y / 2,
                    y2: -y / 2,
                    textPosition: textOffset + primaryTickLength / 2,
                };
            case 'inner':
                return {
                    y1: 0,
                    y2: -y,
                    textPosition: textOffset,
                };
            case 'outer':
                return {
                    y1: 0,
                    y2: y,
                    textPosition: textOffset + primaryTickLength,
                };
            default:
                return {
                    y1: 0,
                    y: 0,
                    textPosition: 0,
                };
        }
    }
    const gridLinesElement = displayPrimaryGridLines ? (_jsx(HorizontalAxisGridLines, { position: position, plotHeight: plotHeight, primaryTicks: primaryTicks, style: primaryGridLineStyle }, void 0)) : null;
    const primaryTicksElement = !hidden && !hiddenTicks ? (_jsx(Ticks, { anchor: "middle", primaryTicks: primaryTicks, getPositions: getTickPosition, labelStyle: tickLabelStyle, style: primaryTickStyle, dominantBaseline: isBottom ? 'text-before-edge' : 'text-after-edge' }, void 0)) : null;
    const axisLineElement = !hidden && !hiddenLine ? (_jsx(HorizontalAxisLine, { style: lineStyle, plotWidth: plotWidth }, void 0)) : null;
    const labelElement = !hidden && label ? (_jsx(HorizontalAxisLabel, { plotWidth: plotWidth, label: label, labelStyle: labelStyle, verticalAlign: isBottom ? 'start' : 'end' }, void 0)) : null;
    const bboxRef = useContext(bboxContext);
    return (_jsxs("g", { transform: transform, children: [gridLinesElement, _jsxs("g", { ref: bboxRef, children: [_jsx("g", { ref: ticks.ref, children: primaryTicksElement }, void 0), _jsx("g", { ref: axisRef, children: axisLineElement }, void 0), _jsx("g", { transform: `translate(0, ${ticks.height * (isBottom ? 1 : -1)})`, children: labelElement }, void 0)] }, void 0)] }, void 0));
}
//# sourceMappingURL=HorizontalAxis.js.map