import { ScaleOrdinal, ScaleContinuousNumeric, ScaleLinear, ScaleLogarithmic } from 'd3-scale';
import { Dispatch } from 'react';
import type { AxisProps } from './components/Axis/Axis';
import { LegendPosition } from './components/Legend';
import type { ActionType, Position, SeriesPoint, TickLabelFormat, VerticalPosition } from './types';
interface PlotSeriesStateAxis {
    min: number;
    max: number;
    axisId: string;
}
export interface PlotSeriesState {
    id: string;
    x: PlotSeriesStateAxis;
    y: PlotSeriesStateAxis;
    label: string;
    data?: SeriesPoint[];
}
export interface PlotState {
    headingPosition: VerticalPosition | null;
    legendPosition: LegendPosition | null;
    legendMargin: number;
    series: PlotSeriesState[];
    axis: Record<string, PlotStateAxis>;
}
declare type PlotStateAxis = Pick<AxisProps, 'position' | 'min' | 'max' | 'paddingStart' | 'paddingEnd' | 'flip' | 'scale' | 'tickLabelFormat'>;
export declare type PlotReducerActions = ActionType<'newData', PlotSeriesState> | ActionType<'removeData', {
    id: string;
}> | ActionType<'newAxis', {
    id: string;
} & PlotStateAxis> | ActionType<'removeAxis', {
    id: string;
}> | ActionType<'addHeading', {
    position: VerticalPosition;
}> | ActionType<'removeHeading'> | ActionType<'addLegend', {
    position: LegendPosition;
    margin: number;
}> | ActionType<'removeLegend'>;
interface PlotAxisContextGeneric<Scale extends ScaleContinuousNumeric<number, number>> {
    scale: Scale;
    tickLabelFormat: TickLabelFormat;
    position: Position;
}
export declare type PlotAxisContext = ({
    type: 'linear';
} & PlotAxisContextGeneric<ScaleLinear<number, number>>) | ({
    type: 'log';
} & PlotAxisContextGeneric<ScaleLogarithmic<number, number>>);
export interface PlotContext {
    width?: number;
    height?: number;
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
    plotWidth: number;
    plotHeight: number;
    labels?: Array<{
        id: string;
        label: string;
    }>;
    colorScaler: ScaleOrdinal<string, string>;
    axisContext: Record<string, PlotAxisContext>;
}
export declare function plotReducer(state: PlotState, action: PlotReducerActions): void;
declare type PlotDispatch = Dispatch<PlotReducerActions>;
export declare const plotContext: import("react").Context<PlotContext>;
export declare const plotDispatchContext: import("react").Context<PlotDispatch>;
export declare function usePlotContext(): PlotContext;
export declare function usePlotDispatchContext(): PlotDispatch;
interface SizeProps {
    plotWidth: number;
    plotHeight: number;
}
export declare function useAxisContext(state: PlotState, { plotWidth, plotHeight }: SizeProps): Record<string, PlotAxisContext>;
export {};
