"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const react_d3_utils_1 = require("react-d3-utils");
const bboxContext_1 = require("../../bboxContext");
const HorizontalAxisGridLines_1 = __importDefault(require("./HorizontalAxisGridLines"));
const HorizontalAxisLabel_1 = __importDefault(require("./HorizontalAxisLabel"));
const HorizontalAxisLine_1 = __importDefault(require("./HorizontalAxisLine"));
const Ticks_1 = require("./Ticks");
function HorizontalAxis(props) {
    const { primaryTickStyle, primaryTickLength, tickPosition, hiddenTicks, primaryGridLineStyle, lineStyle, hiddenLine, hidden, plotWidth, plotHeight, displayPrimaryGridLines, label, labelStyle, axisRef, primaryTicks, position, tickLabelStyle, } = props;
    const isBottom = position === 'bottom';
    const textOffset = 0;
    const transform = isBottom ? `translate(0, ${plotHeight})` : undefined;
    const ticks = (0, react_d3_utils_1.useBBoxObserver)();
    function getTickPosition(value) {
        const { y1, y2, textPosition } = getTickY();
        return {
            line: {
                x1: value,
                x2: value,
                y1: y1,
                y2: y2,
            },
            text: {
                x1: value,
                y1: isBottom ? textPosition : -textPosition,
            },
        };
    }
    function getTickY() {
        const y = isBottom ? primaryTickLength : -primaryTickLength;
        switch (tickPosition) {
            case 'center':
                return {
                    y1: y / 2,
                    y2: -y / 2,
                    textPosition: textOffset + primaryTickLength / 2,
                };
            case 'inner':
                return {
                    y1: 0,
                    y2: -y,
                    textPosition: textOffset,
                };
            case 'outer':
                return {
                    y1: 0,
                    y2: y,
                    textPosition: textOffset + primaryTickLength,
                };
            default:
                return {
                    y1: 0,
                    y: 0,
                    textPosition: 0,
                };
        }
    }
    const gridLinesElement = displayPrimaryGridLines ? ((0, jsx_runtime_1.jsx)(HorizontalAxisGridLines_1.default, { position: position, plotHeight: plotHeight, primaryTicks: primaryTicks, style: primaryGridLineStyle }, void 0)) : null;
    const primaryTicksElement = !hidden && !hiddenTicks ? ((0, jsx_runtime_1.jsx)(Ticks_1.Ticks, { anchor: "middle", primaryTicks: primaryTicks, getPositions: getTickPosition, labelStyle: tickLabelStyle, style: primaryTickStyle, dominantBaseline: isBottom ? 'text-before-edge' : 'text-after-edge' }, void 0)) : null;
    const axisLineElement = !hidden && !hiddenLine ? ((0, jsx_runtime_1.jsx)(HorizontalAxisLine_1.default, { style: lineStyle, plotWidth: plotWidth }, void 0)) : null;
    const labelElement = !hidden && label ? ((0, jsx_runtime_1.jsx)(HorizontalAxisLabel_1.default, { plotWidth: plotWidth, label: label, labelStyle: labelStyle, verticalAlign: isBottom ? 'start' : 'end' }, void 0)) : null;
    const bboxRef = (0, react_1.useContext)(bboxContext_1.bboxContext);
    return ((0, jsx_runtime_1.jsxs)("g", { transform: transform, children: [gridLinesElement, (0, jsx_runtime_1.jsxs)("g", { ref: bboxRef, children: [(0, jsx_runtime_1.jsx)("g", { ref: ticks.ref, children: primaryTicksElement }, void 0), (0, jsx_runtime_1.jsx)("g", { ref: axisRef, children: axisLineElement }, void 0), (0, jsx_runtime_1.jsx)("g", { transform: `translate(0, ${ticks.height * (isBottom ? 1 : -1)})`, children: labelElement }, void 0)] }, void 0)] }, void 0));
}
exports.default = HorizontalAxis;
