"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Legend = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
/* eslint-disable react/no-array-index-key */
const react_1 = require("react");
const react_d3_utils_1 = require("react-d3-utils");
const legendOffsetContext_1 = require("../contexts/legendOffsetContext");
const legendContext_1 = require("../legendContext");
const plotContext_1 = require("../plotContext");
const Markers_1 = require("./Markers");
function exclusiveProps(offsets, a, b, position) {
    if (offsets[a] !== undefined) {
        if (offsets[b] !== undefined) {
            throw new Error(`${a} and ${b} should't be both defined for the position ${position}`);
        }
        return { key: a, value: offsets[a] };
    }
    return offsets[b] !== undefined ? { key: b, value: offsets[b] } : {};
}
function translation(position, legendOffsets, plotWidth, plotHeight, legendOffset) {
    switch (position) {
        case 'embedded': {
            const { key: verticalKey = 'top', value: verticalValue = 10 } = exclusiveProps(legendOffsets, 'top', 'bottom', position);
            const { key: horizontalKey = 'left', value: horizontalValue = 10 } = exclusiveProps(legendOffsets, 'left', 'right', position);
            const x = horizontalKey === 'right'
                ? plotWidth - horizontalValue
                : horizontalValue;
            const y = verticalKey === 'bottom' ? plotHeight - verticalValue : verticalValue;
            return {
                x,
                y,
                horizontalAlign: horizontalKey === 'left' ? 'start' : 'end',
                verticalAlign: verticalKey === 'top' ? 'start' : 'end',
            };
        }
        case 'top': {
            const { value: horizontalValue = plotWidth / 2 } = exclusiveProps(legendOffsets, 'left', 'right', position);
            const x = horizontalValue;
            const y = (-legendOffsets.bottom || 0) - legendOffset;
            return { x, y, horizontalAlign: 'middle', verticalAlign: 'end' };
        }
        case 'right': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendOffsets, 'top', 'bottom', position);
            const y = verticalKey === 'bottom' ? -verticalValue : verticalValue;
            const x = plotWidth + (legendOffsets.left || 0) + legendOffset;
            return { x, y, horizontalAlign: 'start', verticalAlign: 'middle' };
        }
        case 'bottom': {
            const { value: horizontalValue = plotWidth / 2 } = exclusiveProps(legendOffsets, 'left', 'right', position);
            const x = horizontalValue;
            const y = plotHeight + (legendOffsets.top || 0) + legendOffset;
            return { x, y, horizontalAlign: 'middle', verticalAlign: 'start' };
        }
        case 'left': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendOffsets, 'top', 'bottom', position);
            const y = verticalKey === 'bottom' ? -verticalValue : verticalValue;
            const x = (-legendOffsets.right || 0) - legendOffset;
            return { x, y, horizontalAlign: 'end', verticalAlign: 'middle' };
        }
        default: {
            throw new Error(`Position ${JSON.stringify(position)} unknown`);
        }
    }
}
function Legend({ position, margin = 10, ...legendOffsets }) {
    const { plotWidth, plotHeight } = (0, plotContext_1.usePlotContext)();
    const plotDispatch = (0, plotContext_1.usePlotDispatchContext)();
    const [state] = (0, legendContext_1.useLegend)();
    const legendOffset = (0, react_1.useContext)(legendOffsetContext_1.legendOffsetContext);
    const alignGroupProps = (0, react_1.useMemo)(() => {
        return translation(position, legendOffsets, plotWidth, plotHeight, legendOffset);
    }, [position, legendOffsets, plotWidth, plotHeight, legendOffset]);
    (0, react_1.useEffect)(() => {
        plotDispatch({ type: 'addLegend', payload: { position, margin } });
        return () => plotDispatch({ type: 'removeLegend' });
    }, [plotDispatch, position, margin]);
    return ((0, jsx_runtime_1.jsx)(react_d3_utils_1.AlignGroup, { ...alignGroupProps, children: state.labels.map((value, index) => {
            const xPos = 10;
            const yPos = (index + 1) * 16 - xPos + 5;
            if (value.range) {
                return ((0, jsx_runtime_1.jsxs)("g", { transform: `translate(${xPos}, ${0})`, children: [getRangeShape({
                            index,
                            rangeColor: value.range.rangeColor,
                            lineColor: value.colorLine,
                        }), (0, jsx_runtime_1.jsx)("text", { x: 30, y: `${index + 1}em`, children: value.label }, `text-${value.label}-${index}`)] }, `${value.colorLine}/${value.range.rangeColor}-${value.label}`));
            }
            const Marker = Markers_1.markersComps[value.shape.figure];
            return ((0, jsx_runtime_1.jsxs)("g", { transform: `translate(${xPos}, ${0})`, children: [getLineShape({ index, color: value.colorLine }), (0, jsx_runtime_1.jsx)("g", { transform: `translate(${xPos - 1}, ${yPos})`, children: !value.shape.hidden && ((0, jsx_runtime_1.jsx)(Marker, { size: 10, style: { fill: value.shape.color } }, void 0)) }, void 0), (0, jsx_runtime_1.jsx)("text", { x: 30, y: `${index + 1}em`, children: value.label }, `text-${value.label}-${index}`)] }, `${value.colorLine}/${value.shape.color}-${value.label}`));
        }) }, void 0));
}
exports.Legend = Legend;
function getLineShape(config) {
    const x = 0;
    const y = (config.index + 1) * 16 - x - 7;
    return (0, jsx_runtime_1.jsx)("rect", { x: x, y: y, width: "20", height: "3", fill: config.color }, void 0);
}
function getRangeShape(config) {
    const x = 0;
    const y = (config.index + 1) * 16 - x - 12;
    return ((0, jsx_runtime_1.jsxs)("g", { transform: `translate(${x}, ${y})`, children: [(0, jsx_runtime_1.jsx)("rect", { width: "20", height: "10", fill: config.rangeColor }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: 0, y: 0, x2: 20, y2: 0, stroke: config.lineColor, strokeWidth: 3 }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: 0, y1: 10, x2: 20, y2: 10, stroke: config.lineColor, strokeWidth: 3 }, void 0)] }, void 0));
}
