"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePlotSizes = void 0;
const react_1 = require("react");
function usePlotSizes({ width, height, margin, topAxisHeight, rightAxisWidth, bottomAxisHeight, leftAxisWidth, headingPosition, headingHeight, legendPosition, legendMargin, legendWidth, legendHeight, }) {
    const { top = 0, right = 0, bottom = 0, left = 0 } = margin;
    return (0, react_1.useMemo)(() => {
        let plotWidth = width - left - leftAxisWidth - right - rightAxisWidth;
        let plotHeight = height - top - headingHeight - topAxisHeight - bottom - bottomAxisHeight;
        let leftOffset = left + leftAxisWidth;
        let topOffset = top + topAxisHeight;
        if (headingPosition === 'top') {
            topOffset += headingHeight;
        }
        const totalLegendHeight = legendHeight + legendMargin;
        const totalLegendWidth = legendWidth + legendMargin;
        let legendOffset = 0;
        switch (legendPosition) {
            case 'top':
                plotHeight -= totalLegendHeight;
                topOffset += totalLegendHeight;
                legendOffset = topAxisHeight + legendMargin;
                break;
            case 'right':
                plotWidth -= totalLegendWidth;
                legendOffset = rightAxisWidth + legendMargin;
                break;
            case 'bottom':
                plotHeight -= totalLegendHeight;
                legendOffset = bottomAxisHeight + legendMargin;
                break;
            case 'left':
                plotWidth -= totalLegendWidth;
                leftOffset += totalLegendWidth;
                legendOffset = leftAxisWidth + legendMargin;
                break;
            default:
                break;
        }
        return { plotWidth, plotHeight, leftOffset, topOffset, legendOffset };
    }, [
        width,
        height,
        top,
        right,
        bottom,
        left,
        topAxisHeight,
        rightAxisWidth,
        bottomAxisHeight,
        leftAxisWidth,
        headingPosition,
        headingHeight,
        legendPosition,
        legendMargin,
        legendWidth,
        legendHeight,
    ]);
}
exports.usePlotSizes = usePlotSizes;
